import TAP
import threading
from threading import Thread
from DTester import DTester
import sys
import os
import time
import signal
import subprocess
import logging

try:
    set
except NameError:
    from sets import Set as set, Immutable as frozenset

class UnknownBarrier(Exception):
    "Used when requesting an unknown barrier"
    
class DTestMaster(Thread):
    # configure DTestMaster specific logger
    logger = logging.getLogger("DTestMaster")
    sh  = logging.StreamHandler()
    fmt = logging.Formatter(fmt="## [%(threadName)s]|%(name)s::%(levelname)s:: %(message)s")
    sh.setFormatter(fmt)
    logger.addHandler(sh)
    def __init__(self):
        super(DTestMaster,self).__init__(name="DTestMaster");
        self.builder   = TAP.Builder.create()
        self.barriers  = {}
        self.dtesters  = set()
        self.startTime = 0
        self.endTime   = 0
        self.runningDTesters = 0
        self.nb_steps  = 0
        self.timeout   = None
        
    def register(self, dtester):
        self.logger.info("Registering <" + dtester.getName() + ">...")
        # Add DTester to the set of DTester
        self.dtesters.add(dtester)
        # Link myself to the DTester
        # since DTester will call me back for ok, barrier etc...
        dtester.setDTestMaster(self)
        #self.registerForBarrier(dtester,"StartTest")
        #self.registerForBarrier(dtester,"EndTest")
        self.logger.info("<" + dtester.getName() + "> has registered.")

    def registerForBarrier(self, dtester, barrierId):
        self.logger.info("Barrier <"+ barrierId+ "> registered for DTester <"+ dtester.getName()+ ">")
        if not self.barriers.has_key(barrierId):
            self.barriers[barrierId]= dict()
            self.barriers[barrierId]['init']    = set()
            self.barriers[barrierId]['reached'] = set()
            self.barriers[barrierId]['barrier'] = threading.Event()
        self.barriers[barrierId]['init'].add(dtester)
        self.barriers[barrierId]['reached'].add(dtester)
               
    def ok(self, dtester, *args, **kwargs):
        self.builder.ok(*args,**kwargs)
        return 0

    def setGlobalTimeout(self, timeout):
        self.timeout = timeout

    def globalTimeOut(self):
        self.logger.fatal("Global Time out, exiting")
        os._exit(1)
        #sys.exit(1)
        
    def barrier(self, dtester, barrierId, timeout):
        # check whether the barrier has been registered
        if not self.barriers.has_key(barrierId):
            raise UnknownBarrier, "barrier ID ="+barrierId
        #
        try:
            self.barriers[barrierId]['reached'].remove(dtester)
        except KeyError:
            self.logger.warning("DTester <"+ dtester.getName()+ " does not belong to barrier.")
            return
        
        self.logger.info("DTester < "+ dtester.getName()+ "> entered barrier <" + barrierId + ">.")
        if len(self.barriers[barrierId]['reached']) == 0:
            self.builder.ok(True,desc="Barrier <%s> crossed by all <%d> registered DTester(s)" % (barrierId, len(self.barriers[barrierId]['init']))) 
            self.barriers[barrierId]['barrier'].set()
        else:
            self.barriers[barrierId]['barrier'].wait(timeout)
            if (not self.barriers[barrierId]['barrier'].isSet()):
                self.builder.ok(False,desc="Barrier <%s> timed-out for DTester <%s> waiting no more than <%f seconds>" % (barrierId,dtester.getName(),timeout))
                # re-add myself since I did timed-out
                self.barriers[barrierId]['reached'].add(dtester)
                # relieve other to cross the barrier.
                # FIXME is there a race condition on others which make them
                # FIXME believe they timed-out too??
                self.barriers[barrierId]['barrier'].set()
            else:
                self.logger.info("DTester < "+ dtester.getName()+ "> crossed barrier <" + barrierId + ">.") 

    def run(self):

        # set up global time out
        if self.timeout != None:
            t = threading.Timer(self.timeout,self.globalTimeOut)
            t.start()
            
        # Initialize all registered dtesters
        for dtester in self.dtesters:
            self.logger.info("Initializing <"+ dtester.getName()+ ">...")
            dtester.initialize()
            self.nb_steps += dtester.nb_steps

        self.logger.debug("Defined %d barriers" % len(self.barriers))
        self.nb_steps += len(self.barriers)

        # plan test 
        self.builder.set_plan(self.nb_steps, None)
        
        # Start all registered dtesters
        for dtester in self.dtesters:
            self.logger.info("Starting <"+ dtester.getName()+ ">...")
            dtester.start()
            self.runningDTesters += 1

        self.logger.info("Now <%d> registered DTesters launched.",self.runningDTesters)

        # Wait for DTesters to terminate
        self.joinedDTesters = set()
        while self.runningDTesters > 0:
            for dtester in self.dtesters:
                if dtester.isAlive():
                    time.sleep(1)
                else:                    
                    dtester.join()
                    self.logger.info("joined <"+ dtester.getName()+ ">.")
                    self.joinedDTesters.add(dtester)
                    dtester.stdout.flush()
                    dtester.stderr.flush()
                    self.runningDTesters -= 1;
            # use the powerful difference_update on set
            # to continue looping on remaining (not already joined dtesters)
            self.dtesters.difference_update(self.joinedDTesters)

        if self.timeout !=None:
            t.cancel()
            
        # Report DTesters run
        ##for dtester in self.joinedDTesters:
##            self.logger.debug("Begin-of stdout from <"+ dtester.getName()+ ">")
##            for line in dtester.stdout:
##                self.logger.debug(line)
##            self.logger.debug("-End-of stdout- <"+ dtester.getName()+ ">")

##            self.logger.debug("Begin-of stderr <"+ dtester.getName()+ ">")
##            for line in dtester.stderr:
##                self.logger.debug(line)
##            self.logger.debug("-End-of stderr- <"+ dtester.getName()+ ">")

    def startTestSequence(self):
        self.start()

    def waitTestSequenceEnd(self):
        self.join()
    
    
