#!/usr/bin/env python

import dtest
import logging
import os
import time
import threading
import getopt, sys

def usage():
    print "Usage:\n %s --provider=<user>@[<host>]:<stub_path> --consumer=<user>@[<host>]:<stdout_path>" % sys.argv[0]

def getUserHostPath(argument):
    if argument.find("@") != -1:
        (user,argument) = argument.split("@",1)
    else:
        user = os.environ["USER"]
    if argument.find(":") != -1:
        (host,path) = argument.split(":",1)
    else:
        host = "localhost"
        path = argument
    retval = dict()
    retval['user'] = user
    retval['host'] = host
    retval['path'] = path
    return retval

try:
    opts, args = getopt.getopt(sys.argv[1:], "p:c:", ["provider=", "consumer="])
except getopt.GetoptError, err:
    print >> stderr, "opt = %s, msg = %s" % (err.opt,err.msg)
    usage()
    sys.exit(2)

if len(opts) < 2:
    usage()
    sys.exit(2)

for o, a in opts:
    if o in ("-p", "--provider"):
        stub_param   = getUserHostPath(a)
    if o in ("-c", "--consumer"):
        stdout_param = getUserHostPath(a)

##stub_path="/home/eric/TSP/builds/tsp_oncrpc/Linux/Debug/bin/tsp_stub_server"
##stdout_path="/home/eric/TSP/builds/tsp_oncrpc/Linux/Debug/bin/tsp_stdout"
##stub_path = "$HOME/TSP/build_i386/Linux/Debug/bin/tsp_stub_server"
##stdout_path = "$HOME/TSP/build_i386/Linux/Debug/bin/tsp_stdout_client"

# initialize your (distributed testers)
#stdout  = dtest.DTesterLocal("tsp_stdout")
stdout  = dtest.DTesterSSH("tsp_stdout",stdout_param['user'],host=stdout_param['host'])
#stub    = dtest.DTesterSSH("tsp_stubbed_server")
stub    = dtest.DTesterSSH("tsp_stubbed_server",stub_param['user'],host=stub_param['host'])
global_timeout = 10

dtest.DTester.logger.setLevel(level=logging.ERROR)

stub.addRunStep("ok",True,"TSP Stub and Stdout Starts")
stub.addRunStep("runCommand",command=stub_param['path'])
stub.addRunStep("expectFromCommand",pattern="TSP Provider on PID",timeout=global_timeout)
stub.addRunStep("barrier","provider started",timeout=global_timeout)
stdout.addRunStep("barrier","provider started",timeout=global_timeout)
stdout.addRunStep("runCommand",command=stdout_param['path']+" -u rpc://"+stub.host)
stdout.addRunStep("expectFromCommand",pattern="tsp_stdout_client: Using provider URL",timeout=5)
stdout.addRunStep("barrier","consumer started",timeout=global_timeout)
stdout.addRunStep("expectFromCommand",pattern="End of Test OK",timeout=global_timeout)
stdout.addRunStep("terminateCommand")
stdout.addRunStep("waitCommandTermination")
stdout.addRunStep("barrier","consumer ended",timeout=1)
stub.addRunStep("barrier","consumer ended",timeout=global_timeout)
stub.addRunStep("terminateCommand")
stub.addRunStep("waitCommandTermination")
stub.addRunStep("ok",True,"TSP Stub and Stdout Ends")

# Here begins the test
dtest.DTestMaster.logger.setLevel(level=logging.WARNING)
dtest.DTester.logger.setLevel(level=logging.WARNING)
myDTestMaster = dtest.DTestMaster()

def goTest():
    myDTestMaster.register(stdout)
    myDTestMaster.register(stub)
    myDTestMaster.setGlobalTimeout(40)
    myDTestMaster.startTestSequence()
    myDTestMaster.waitTestSequenceEnd()
    
goTest()

